#!/usr/bin/perl -w

###########################################################################
#
# translator.pl
#
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the 
# University of Waikato, New Zealand.
#
# Copyright (C) 1999 New Zealand Digital Library Project
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################


BEGIN {
    die "GSDLHOME not set\n" unless defined $ENV{'GSDLHOME'};
    unshift (@INC, "$ENV{'GSDLHOME'}/perllib");
}

use parsargv;
use util;
use Cwd;
use File::Basename;
use GDBM_File;
use CGI;
#require utf8;
#use unicode;

sub main
{
    #gets the name of the translation language
    my $file = shift(@_);
    #gets the name of the base language
    my $baselanguage = shift(@_);
    
    #checks that arguments were supplied
    if (!$file) {
	die "You didn't supply the name of the language file! Aborting...\n";
    }
    if (!$baselanguage) {
	die "You didn't supply the name of the base language file! Aborting...\n";
    }

    #generates a hash of the differences between the base language file and the
    #translation language file, of the form macroname->macrotext
    my ($diffhash) = &initial_hashing($file, $baselanguage);

    #generates HTML code to display user interface on webpage with what needs translation
    #returns an array of CGI data
    my @queries = &generate_pages($diffhash, $file, $baselanguage);

}

sub initial_hashing {

    my ($file, $baselanguage) = @_;

    #catches cases where the macro file name is different to the language name
    $file = "port" if ($file =~ m/portuguese/);
    $file = "indo" if ($file =~ m/indonesian/);

    #saves the directory path to the database files
    $gdbmdir  = "$ENV{'GSDLHOME'}/tmp/lang/gdbmfiles/";
    #creates the directory if it doesn't exist
    if (!-e $gdbmdir) {
	my $store_umask = umask(0000);
	if (! mkdir($gdbmdir, 0777)) {
	    umask($store_umask);
	    die "Couldn't create directory $gdbmdir\n";
	}
	umask($store_umask);
    }
    #saves the directory path to the macro files
    my $macrodir = "$ENV{'GSDLHOME'}/macros/";
    
    # hashes the macro files into form of macroname->macrotext
    my ($englhash) = &parse_macrofile($macrodir.$baselanguage.".dm");
    my ($engl2hash) =  &parse_macrofile($macrodir.$baselanguage."2.dm");
    
    #uses each of the above hashes to write the db files
    &file_to_db($baselanguage, $englhash);
    &file_to_db($baselanguage."2", $engl2hash);
    
    #uses cvs date annotation to assign date of last edit to each macro
    my ($base) = &dateannotation($baselanguage);
    my ($base2) = &dateannotation($baselanguage."2");
    #combines hash to end up with a full hash of the english and english2 macrofiles with
    #each macro having a date allocated to it
    $base = &combine_hashes($base,$base2);
    
    # language macro-filename and database-filename
    my $macrofile = $macrodir.$file.".dm";
    my $dbfile = $gdbmdir.$file.".db";
    
    #if we are creating a new language
    if (! -e $macrofile) {
	
	#hard code of all the packages that exist in the two macrofiles
	#DON'T LIKE THIS CAUSE WHEN A NEW PACKAGE IS ADDED, WILL CREATE ERROR!!!!!!!!!!!!!!!!!!!!!!
	@packages = ("Global", "about", "document", "query", "preferences", "browse", "help",
		     "home", "homehelp", "extlink", "authen", "collector", "docs", "usersedituser",
		     "usersdeleteuser", "bsummary", "status", "users", "gsdl", "userschangepasswd",
		     "userschangepasswdok", "translang");

	#creates a new macro file and writes a framework of a macrofile to it
	open MCROFILE, ">$macrofile" or die "MURGH\n";
	
	foreach $packg (@packages) {
	    print MCROFILE "package $packg\n";
	    print MCROFILE "# text macros\n";
	    print MCROFILE "# icons\n";
	}

	close MCROFILE;

	#get a code for the new language as the first two characters of the language
	$code = $& if($file =~ m/\w\w/);

	#opens the existing file containing the languages and their codes
	open LANGFILE, "<$ENV{'GSDLHOME'}/tmp/lang/package_forms/languages.log" or die "MURGH\n";

	#scrolls through file checking that the new code does not already exist
	while (<LANGFILE>) {
	    #if code does exist, change to 2nd and 3rd characters of new language
	    #THIS CHECKING IS NOT RECURSIVE, SO IF 2ND ONE DID ALREADY EXIST WOULD CAUSE PROBLEMS!!!!!!!
	    if ($_ eq $code){ 
		$code = $& if($file =~ m/.\w\w/);
		$code =~ s/\A.//;
	    }
	}

	close LANGFILE;
	
	#appends the name of the new language and its associated code to the end of the file
	open LANGFILE, ">>$ENV{'GSDLHOME'}/tmp/lang/package_forms/languages.log" or die "MURGH\n";
	print LANGFILE "\n$file";
	$code = $& if($file =~ m/\w\w/);
	print LANGFILE "\n$code";
	close LANGFILE;
    }

    #hashes translation language macrofiles into form of macroname->macrotext
    my ($langhash) = &parse_macrofile($macrofile);
    $macrofile = $macrodir.$file."2.dm";
    my ($langhash2) = &parse_macrofile($macrofile);
     
    #uses these hashes to write the db files for the translation language
    &file_to_db($file, $langhash);
    &file_to_db($file."2", $langhash2);

    #annotates each macro name with the date that it was last edited
    my $foreign = &dateannotation($file);
    my $foreign2 = &dateannotation($file . "2");

    #combines data into final hash of macroname->macrotext where each
    #macro has the date of last edition
    $foreign = &combine_hashes($foreign, $foreign2);
    
    #finds the differences between the english database (base language)
    #and the supplied languages database and hashes these differences
    #in the form macroname->macrotext
    my $diffhash = &db_db_diffs($gdbmdir.$baselanguage.".db", $dbfile, $base, $foreign, $baselanguage);
    $dbfile = $gdbmdir.$file."2.db";
    my $diffhash2 = &db_db_diffs($gdbmdir.$baselanguage."2.db", $dbfile, $base, $foreign, $baselanguage);

    #combines the differences from the first and second language files into one hash
    $diffhash = &combine_diffhash($diffhash,$diffhash2);

    return (\%$diffhash);
}

#combines two existing hashes of the same format together
sub combine_hashes {
    
    my ($hash1, $hash2) = @_;
    my %combined = ();

    foreach $key (sort (keys %$hash1)) {
	$combined{$key} = $hash1->{$key};
    }
    foreach $key (sort (keys %$hash2)) {
	$combined{$key} = $hash2->{$key};
    }

    return (\%combined);

}

#combines to existing diffhashes together, retaining information regarding
#whether a macro came from the first or second hash
sub combine_diffhash {
    
    my ($hash1, $hash2) = @_;
    my %combined = ();

    foreach $key (sort (keys %$hash1)) {
	$combined{$key} = [$hash1->{$key},"1"];
    }
    foreach $key (sort (keys %$hash2)) {
	$combined{$key} = [$hash2->{$key},"2"];
    }

    return (\%combined);

}

#to parse the .dm files into a hash of macronames->macrotext
sub parse_macrofile
{
    #saves the path of the macro file
    my $filename = shift(@_);
    #initialises some local variables
    my $currpackage;
    my %macros = ();
    my @date = localtime(time);
    
    #opens macro file or kills program
    open(IN, "<$filename") or return;
    #reads in contents of macro file, line by line
    while (<IN>) {
	# find out the current package that macros belong to
	if (s/^package //) {
	    $currpackage = $_;
	    chomp($currpackage);
	}
	#if line contains a macro name
	elsif (/^(_\w+_)/) {
	    # gets the name of the macro ($1 contains text matched by corresponding
	    #set of parentheses in the last matched pattern within the dynamic scope
	    #which here matches (_\w+_) )
	    my $macroname = $1;
	    #saves the day/month/year
	    my $macrotext = $date[3]. "/" . $date[4] . "/" . $date[5] ."\n";
	    # key to the hash and the database
	    my $key = $currpackage . "::" . $macroname;
	    # get the first line of the macro
	    $_ =~ s/^_\w+_ *//;
	    # while there is still text of the macro to go...
	    while ($_ !~ /.*\}/) {
		# ... adds it to the macrotext variable
		$macrotext .= $_;
		$_ = <IN>;
	    }
	    $macrotext .= $_;
	    #hashes the macroname and the macrotext
	    $macros{$key} = $macrotext;

	}
	#line in format ## "sometext" ## macro ## macroname ##
	elsif (/^\#\# .*/) {
	    my $macroname = $_;
	    #saves the day/month/year
	    my $macrotext = " ".$date[3]. "/" . $date[4] . "/" . $date[5] ."\n";
	    
	    #if the macro text contains a new line will run over two lines
	    unless ($macroname =~ m/^\#\# .*\#\#/) {
		
		$macroname = $_;
		chomp($macroname);
		$_ = <IN>;
		$macroname .= $_;
		
	    }

	    #split the image macro header on ##
	    my @names = split(/\s*\#\#\s*/, $macroname);
	    #save current contents of macroname into macrotext
	    $macrotext .= $macroname;
	    $_ = <IN>;
	    # overwrite macroname with macroname from ## ... ## ... ## HERE ###
	    $macroname = $names[(scalar @names) - 1];
	    #append package to the front of macroname
	    my $key = $currpackage . "::" . $macroname; # key to the hash and the database

	    #read in the rest of the text associated with the image macro (i.e height | width etc.)
	    while ($_ !~ /^\s+/) {
		$macrotext .= $_;
		$_ = <IN>;
	    }
	    #hashes macroname and macrotext
	    $macros{$key} = $macrotext;
	}
	
    }    
    close IN;

    
    return (\%macros);
}

#to annotate each macro with the date that it was last edited
sub dateannotation {
    
    $language = shift(@_);
    my %macrodated = ();

    #system call which returns one big string containing the whole macro file
    #with each line annotated with the date it was last modified
    $ENV{'CVS_RSH'}="ssh";
    $file = qx/cd $ENV{'GSDLHOME'}\/macros;cvs annotate $language.dm/;
    #split the file into an array of lines
    @file = split(/\n/,$file);
    #return $file variable to empty state to free up memory
    $file = "";
   
    for ($p = 0; $p < scalar(@file); $p++) {
	
	$line = $file[$p];
	chomp($line);
	#cuts the date of the beginning of the line and saves it
	$line =~ s/.*\(.*\):\s//;
	$date = $&;
	
	#checks that date is in the correct format
	$date = $& if($date =~ m/\d\d-\w\w\w-\d\d/);
	
	#if line contains an image macro
	if ($line =~ m/\#\# .*/) {
	    
	    #catch macrotexts which cover two lines
	    unless ($line =~ m/\A\#\# .*\#\#/) {
		$line = $file[++$p];
		chomp($line);
		$line =~ s/.*\(.*\):\s//;
	    }
	    my @icon = split(/\s*\#\#\s*/, $line);
	    
	    $macro = $icon[(scalar @icon) - 1];
	    
	    #hash the macro name and the date
	    $macrodated{$macro} = $date;
	    
	    #read through the rest of the image macro so that the normal macros it
	    #may contain don't get included in the macrodated hash
	    while ($line =~ m/\s+/) {
		$line = $file[++$p];
		chomp($line);
		$line =~ s/.*\(.*\):\s//;
	    }
	}
	#if the line contains a normal macro in _this_ form
	elsif ($line =~ m/\A(_\w+_)/) {
	    $line = $&;
	    #hash the macro name and the date
	    $macrodated{$line} = $date;
	}
	
	    
    }
    
    return (\%macrodated);
}

#used to update the db file when changes occur in the dm file
#will execute this command everytime the translator is entered
#for both the base language and translation language db files
sub file_to_db
{
    my ($filename, $filehash) = @_;

    my $database = "$ENV{'GSDLHOME'}/tmp/lang/gdbmfiles/$filename.db";
    my %dbhash = ();

    
    if (-e $database) {
	
	tie(%dbhash, 'GDBM_File', $database, 1, 0640) or die "$!";

	foreach $filekey (sort keys(%$filehash)) {
	   
	    if (!($dbhash{$filekey})) {
		$dbhash{$filekey} = $filehash->{$filekey};
	    }
	    else {
		# strip off the dates from both entries...
		my $filemacro = ($filehash->{$filekey});
		my $dbmacro = ($dbhash{$filekey});
		$filemacro =~ s/^.*//;
		$dbmacro =~ s/^.*//;

		if ($dbmacro ne $filemacro) {
		    #update the database to reflect the change...
		    $dbhash{$filekey} = $filehash->{$filekey};
		}
	    }
	}
	
    }
    else { # create the database file for the particular language file...
	# tie the hash to the database
	tie(%dbhash, "GDBM_File", $database, GDBM_WRCREAT, 0640);
	
	%dbhash = %$filehash;
    }

    untie %dbhash;
}

#Takes two paramaters which are two database filenames, works out
#the differences between the two, and returns them in %diffhash
#which holds entries which are not in db_one but are in db_two and
#entries which are in both, but the db_one entry has been updated since
#the db_two entry was last edited
sub db_db_diffs
{
    my ($db_one, $db_two, $basedates, $foreigndates, $baselanguage) = @_;

    my %db1hash = ();
    my %db2hash = ();
    my %months = ("Jan",1,"Feb",2,"Mar",3,"Apr",4,"May",5,"Jun",6,
		  "Jul",7,"Aug",8,"Sep",9,"Oct",10,"Nov",11,"Dec",12);
    
    # this hash holds the database entries that are in 
    # english.db but not in $filename.db
    my %diffhash = ();

    #ties the hash to database so when you read from hash it fetches from external
    #database and when you set hash it writes to external database
    tie(%db1hash, "GDBM_File", $db_one, GDBM_READER, 0640);
    tie(%db2hash, "GDBM_File", $db_two, GDBM_READER, 0640);

   
    #the key is the name of the macro
    foreach $key (sort keys(%db1hash)) {

	#if the macro isn't there at all, then it's different
	if (!$db2hash{$key}) {
	    $diffhash{$key} = [$db1hash{$key}, ""];
	}
	#else if the macro is in both, want to compare date stamps of date hashes
	#and if foreign date is older than base date want to add to diffhash aswell
	else {
	    $macro = $key;
	    $macro =~ s/\A.*:://;
	    $bdate = $basedates->{$macro};
	    $fdate = "";
	    #check that the macro has a foreign date
	    if ($foreigndates->{$macro}) {
		$fdate = $foreigndates->{$macro};
		@fdate = split(/-/,$fdate);
		@bdate = split(/-/,$bdate);
	    
		#compare year then month then day
		if ($bdate[2] >= $fdate[2]) {
		    if ($months{$bdate[1]} >= $months{$fdate[1]}) {
			if ($bdate[0] > $fdate[0]) {
			    #get the foreign/translation language text as it currently stands
			     $foreigntext = $db2hash{$key};
			     if ($foreigntext =~ m/\#\# /) {
				 $foreigntext = $& if ($foreigntext =~ m/\"(.|^)*\"/);
			     }
			     else {
				 $foreigntext =~ s/^.*\n//;
				 $foreigntext =~ s/.*\{//;
				 $foreigntext =~ s/\}\Z//;
				 $foreigntext =~ s/\n\Z//;
			     }

			    #store the macroname with the macrotext in the base and foreign languages
			    $diffhash{$key} = [$db1hash{$key}, $foreigntext];
			}
		    }
		}
	    }
	}
    }
    untie %db1hash;
    untie %db2hash;

    return \%diffhash;    
} 

# generates whole website by seperating hash of differences into groups
# of at most 15 macros and then generating a page from each group
sub generate_pages
{
    my ($datahash) = shift(@_);
    my $lang = shift(@_);
    my $baselanguage = shift(@_);
    my %pagehash = ();
    my @query_pages = ();

    # directory in which to put the generated html pages
    my $dir = "$ENV{'GSDLHOME'}/tmp/lang/package_forms/$lang";
   
    # creates directory if it doesn't already exist
    if (!-e $dir) {
	my $store_umask = umask(0000);
	if (! mkdir($dir, 0777)) {
	    umask($store_umask);
	    die "Couldn't create directory $dir\n";
	}
	umask($store_umask);
    }
    
    @pageno = ();
    $pageno = 1;
    $keysperpage = 0;
    $limit = 15;

    # goes through hash of differences between base language and translation
    # sorted first by whether it is from the core or auxiliary macrofile then
    # alphabetically by macroname
    foreach $key (sort {$datahash->{$a}[1] <=> $datahash->{$b}[1]
			    ||
		        $datahash->{$a}[0] cmp $datahash->{$b}[0] } keys(%$datahash)) {
	
	#nested hash. The pagehash contains a hash of all of the translation pages.
	#Then each page is itself contains a hash of all the macros on that page, where
	#each macro is associated with some macrotext and whether it was from the core
	#or auxiliary macrofile
	$pagehash->{$pageno}->{$key} = [$datahash->{$key}[0],$datahash->{$key}[1]];
	$keysperpage++;
	
	#reduce the number of macros per page in these cases
	$limit = 7 if($key =~ m/collector::_text.*/);
	$limit = 7 if($key =~ m/help::_text.*/);
	$limit = 1 if($key =~ m/translang::_textinitial_/);
	$limit = 3 if($key =~ m/home::.*/);
	$limit = 3 if($key =~ m/gsdl::.*/);
	$limit = 1 if($key =~ m/help::_(.*)texthelp.*/);
	$limit = $keysperpage if ($limit <= $keysperpage);

	#if have enough keys to generate a page with
	if ($keysperpage == $limit) {
	    #resets key counting variable
	    $keysperpage = 0;
	    $limit = 15;
	    #opens a file to write the HTML code to for current page
	    open HTMLFILE, ">$dir/$pageno.lang";

	    #passes hash of keys for current page and filehandle, returns CGI data
	    my $query = &generate_form($pagehash->{$pageno}, \*HTMLFILE, $lang, $baselanguage);
	    close HTMLFILE;

	    #push the page number onto the array of pages
	    push(@pageno, $pageno);
	    #increment the current page
	    $pageno++;
	}
    }
    
    #if you exit the loop with keys still to write
    if ($keysperpage != 0) {
	
	#writes out the remaining keys, same format as above
	open HTMLFILE, ">$dir/$pageno.lang";
	my $query = &generate_form($pagehash->{$pageno}, \*HTMLFILE, $lang, $baselanguage);
	close HTMLFILE;
	push(@pageno, $pageno);
    }

    #writes each page number to a file
    open PAGELOG, ">$dir/pageno.log" or die("MURGH\n");
    foreach $page (@pageno) {
	print PAGELOG $page, "\n";
    }
    close PAGELOG;

    #write thankyou page for language translator once translation is complete
    open THANKYOU, ">$dir/thankyou.lang" or die("MURGH\n");
    print THANKYOU "<center> _textthanks_ $lang _texttrans_ <p></center>\n";
    close THANKYOU;

    #returns CGI data
    return @queries;
}

sub generate_form
{
    my $formhash = shift(@_);
    my $fh = shift(@_);
    my $language = shift(@_);
    my $baselanguage = shift(@_);
    # common gateway interface for writing the stuff to the web
    my $query = new CGI;
    my $keynamecount = 1;

    $first = "true";

    foreach $key (sort {$formhash->{$a}[1] <=> $formhash->{$b}[1]} (keys(%$formhash))) {

	my $text = $formhash->{$key}[0][0];

	# whole lot of formatting on strings
	# escape all the '_' with '\' so that Greenstone doesn't substitute
	# the macro names for the contents of the macro itself
	my $keyname = $key;
	  
	$keyname  =~ s/_/\\_/g;
	# strip off the date
	$text =~ s/.*//;
	$text =~ s/(\[l=.*\])//;
	
        # must take care of image macros single and multiple line ones
	$text =~ s/(\<br\>\#\#.*\#\#\s*\<br\>).*/$1/g;
	
	# format the text for displaying in the browser
	$text =~ s/[\{\}]//g unless($text =~ m/\#\# /);

	$text =~ s/(\A\n)//;
	$text =~ s/(\n\Z)//;
	
	
	$keynamecount = 1;
	
	#gets the translation language text which will exist when base macro
	#has been edited after the last edition of the translation macro
	my $default = $formhash->{$key}[0][1];
	#tells whether is a core or auxiliary macro
	my $priority = $formhash->{$key}[1];
	
	#if is a core macro, and the first on the page, table header displays core
	if($priority eq "1" and $first eq "true") {

	    print $fh ($query->hidden(-name=>"tlng",-default=>"$language"),
		       $query->hidden(-name=>"bl",-default=>"$baselanguage"),
		       "</center><img src=\"_httpimg_/core.gif\">",
		       "<table cellpadding=\"10\">\n",
		       "<tr><td><strong><center>".uc ($baselanguage)."</strong></center></td>\n",
		       "<td><strong><center>". uc ($language)."</strong></center></td></tr>\n");
	    #shows that have seen first macro of core type
	    $first = "been";
	}
	#if is a auxiliary macro, and the first on the page, table header displays auxiliary
	if($priority eq "2" and $first eq "true") {
	  print $fh ($query->hidden(-name=>"tlng",-default=>"$language"),
		     $query->hidden(-name=>"bl",-default=>"$baselanguage"),
		     "</center><img src=\"_httpimg_/auxiliary.gif\">",
		     "<table cellpadding=\"10\">\n",
		     "<tr><td><strong><center>". uc ($baselanguage)."</strong></center></td>\n",
		     "<td><strong><center>". uc ($language)."</strong></center></td></tr>\n");
	    $first = "false";
	}	
	#if another macro has already been seen, but this is the first occurence of
	#an auxiliary macro, will finish current table and begin new auxiliary table underneath
	if($priority eq "2" and $first eq "been") {
	    print $fh ("</table><br></center><img src=\"_httpimg_/auxiliary.gif\">",
		       "<table cellpadding=\"10\">",
		       "<tr><td><strong><center>".uc ($baselanguage)."</strong></center></td>\n",
		       "<td><strong><center>". uc ($language)."</strong></center></td></tr>\n");
	    $first = "false";
	}

        # so that when it is an image only get the text to translate not coding stuff aswell
	if ($text =~ m/\#\# (^|.)*/) {
	    if ($& =~ m/".*"/) {
		#prints to the file two text fields, the left one contains the base language
		#macro text, the right contains the translation language macro text if any exists

		my $textbox1;
		if (1) {
		    $rows = 1 if ($rows < 1);
		    $textbox1 = "<textarea name=\"whocaresh\" rows=\"$rows\" cols=\"50\" readonly=\"1\">";
		    $textbox1 .= "$& </textarea>\n";
		}
		else {
		    $textbox1 = $query->textfield(-name=>"whocares",
						 -default=>"$&",
						 -size=>50,
						 -readonly=>1);
		}

		print $fh ("<tr><td align=center>", $textbox1,
			   "</td><td align=center>",
			   $query->hidden(-name=>"$keyname" . "$keynamecount",-default=>"$`"));
		$keynamecount++;

		my $textbox2;
		if (1) {
		    $textbox2 = "<textarea name=\"$keyname$keynamecount\" rows=\"$rows\" cols=\"50\">";
		    $textbox2 .= "$default </textarea><br>\n";  
		}
		else {
		    $textbox2 = $query->textfield(-name=>"$keyname" . "$keynamecount",
						  -default=>"$default",
						  -size=>50);
		}
		print $fh $textbox2;
		$keynamecount++;
		print $fh ($query->hidden(-name=>"$keyname" . "$keynamecount",-default=>"$'"),
			   "</td></tr>\n");
	    }
	}
	elsif ($text =~ m/\S+/) {
		$text =~ s/\s+/ /g;
		
		my @words = split(/ /, $text);
		my $words = scalar(@words);
		my $rows = sprintf("%.0f", $words/5);
		print $fh ("<tr><td align=center>\n");
		#determines how many rows of text are required to display the base language
		#macro text then use this value to determine whether we should use a text area
		#or a text field

		# Experimenting with textfield and text area
		# for now use exclusively textareas.

		# if ($rows >= 0) { 
		if (1) { 
		    $rows = 1 if ($rows < 1);

		    #prints to the file two text areas, the left one contains the base language
		    #macro text, the right contains the translation language macro text if any exists
		    
		    $text =~ s/<p>/<p>\n/g;
		    $text =~ s/<br>/<br>\n/g;
		    $text =~ s/<\/td>/<\/td>\n/g;

		    print $fh ("<textarea name=\"whocaresh\" rows=\"$rows\" cols=\"50\" readonly=\"1\">",
			       "$text </textarea>\n", 
			       "</td><td align=center>\n",
			       "<textarea name=\"$keyname\" rows=\"$rows\" cols=\"50\">", 
			       "$default </textarea><br></td></tr>\n");  
		    
		    #### print STDERR "$text \n";
		}
		else {
		    #prints to the file two text fields, the left one contains the base language
		    #macro text, the right contains the translation language macro text if any exists
		    print $fh ($query->textfield(-name=>"whocares",
						 -default=>"$text",
						 -size=>50,
						 -readonly=>1),
			       "</td><td align=center>\n",
			       $query->textfield(-name=>"$keyname" . "$keynamecount",
						 -default=>"$default",
						 -size=>50),
			       "<br></td></tr>\n");
		}
	    }
    }
    #finishes table and adds a SUBMIT CHANGES option on the end
    print $fh ("</table><br><center>\n",
	       "<table><tr><td align=left width=\"50%\">",
	       $query->reset("RESET FORM"),
	       "</td><td align=right width=\"50%\">",
	       $query->submit($pageno, "_textsubmit_"),
	       "</td></tr><tr><td width=\"50%\"></td><td align=right width=\"50%\">",
	       "<br> _textsubmittext_ \n",
	       "</td></tr></table>");

    # adds the CGI area just written to the array of CGI areas
    push (@queries, $query);

    return $query;
}

&main(@ARGV);

