package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

import java.util.HashMap;
import java.io.File;

/** action class for handling applets */
public class AppletAction extends Action {
    

    public Element process (Element message) {
	
	Element request = (Element)GSXML.getChildByTagName(message, GSXML.REQUEST_ELEM);
	
	// get the collection and service params
	Element cgi_param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(cgi_param_list, false);

	// request_type is display (d) or request (r)
	String request_type = (String)params.get(GSCGI.REQUEST_TYPE_ARG);
	if (!request_type.equals("d")&&!request_type.equals("r")) {
	    System.err.println("AppletAction Error: the rt arg should be either d or r, instead it was "+request_type+"!");
	    return null;
	}
	
	String collection = (String)params.get(GSCGI.COLLECTION_ARG);
	boolean coll_specified = true;
	String service_name=(String)params.get(GSCGI.SERVICE_ARG);
	String lang = request.getAttribute(GSXML.LANG_ATT);
	String to=null;
	if (collection==null||collection.equals("") ) {
	    coll_specified = false;
	    to = service_name;
	} else {
	    to = GSPath.appendLink(collection, service_name);
	}

	if (request_type.equals("r")) {
	    // we are processing stuff for the applet send a message to the service, type="query", and take out the something element, and return that as our result - the applet must take xml

	    Element mr_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	    Element mr_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_PROCESS, to, lang); 
	    mr_message.appendChild(mr_request);
	    // just append all the params for now - should filter out unneeded ones
	    mr_request.appendChild(doc_.importNode(cgi_param_list, true));
       
	    // process the request
	    Element mr_response = (Element)mr_.process(mr_message);
	    // get the applet data out and pass it back as is. 
	    String path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.APPLET_DATA_ELEM);
	    Element applet_info = (Element)GSXML.getNodeByPath(mr_response, path).getFirstChild();
	    return applet_info;
	    
	}
	
	// else request type  = 'd'
	// find the stylesheet
	String stylesheet = GSFile.stylesheetFile(config_, "applet.xsl");
	
	if (stylesheet==null) {
	    System.err.println("AppletAction Error: applet stylesheet not found!");
	    return null;
	}
 
	// we are just displaying the applet - get the description from
	// the service, and process using xslt
	
	
	// create the return page tree
	Element page = doc_.createElement(GSXML.PAGE_ELEM);
	page.setAttribute(GSXML.LANG_ATT, lang);
	// add the lang stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.DISPLAY_ELEM), true));
	// add the config stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.CONFIGURATION_ELEM), true));

	// add in the page Request
	Element page_request = GSXML.duplicateWithNewName(doc_, request, "pageRequest", true);
	page.appendChild(page_request);

	// get the applet description, and the collection info if a collection is specified

	Element mr_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element applet_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_DESCRIBE, to, lang); 
	mr_message.appendChild(applet_request);

	if (coll_specified) {
	    Element coll_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_DESCRIBE, collection, lang); 
	    mr_message.appendChild(coll_request);
	}
	
	Element mr_response = (Element)mr_.process(mr_message);
	    
	Element page_response = doc_.createElement(GSXML.PAGE_RESPONSE_ELEM);
	page.appendChild(page_response);
	// add in the applet info
	String path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.SERVICE_ELEM);
	path = GSPath.appendLink(path, GSXML.APPLET_ELEM);
	Element app_desc = (Element)doc_.importNode(GSXML.getNodeByPath(mr_response, path), true);
	// must handle any params that have values that are not 
	// necessarily known by the service
	editLocalParams(app_desc, config_.library_name_, collection);
	page_response.appendChild(app_desc);
	
	if (coll_specified) {
	    Element coll_response = (Element) mr_response.getElementsByTagName(GSXML.RESPONSE_ELEM).item(1);
	    Element coll_elem = (Element)GSXML.getChildByTagName(coll_response, GSXML.COLLECTION_ELEM);
	    page_response.appendChild(doc_.importNode(coll_elem, true));
	}
	
	// process using the stylesheet
	Document style_doc = converter_.getDOM(new File(stylesheet));
	GSXSLT.absoluteIncludePaths(style_doc, config_);
	return (Element)transformer_.transform(style_doc, page);    
	
    }

    /** this method looks through the PARAMs of the applet description for 
     * 'library' or 'collection'. If found, the params are set to  
     * the appropriate values 
     */
    protected void editLocalParams(Element description, String library_name,
				   String full_collection_name) {

	Node child = description.getFirstChild();
	while (child != null) {
	    if (child.getNodeType() == Node.ELEMENT_NODE) {
		String param = child.getNodeName();
		if (param.equals("PARAM")||param.equals("param")) {
		    String name = ((Element)child).getAttribute("NAME");
		    if (name == null || name.equals("")) {
			// somethings wrong!!
		    } else if (name.equals("library")) {
			((Element)child).setAttribute("VALUE", library_name);
		    } else if (name.equals("collection")) {
			((Element)child).setAttribute("VALUE", full_collection_name);
		    }
		    
		}
	    }
	    child = child.getNextSibling();
	}
    }
	
}
