package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;

// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.Element; 
import org.w3c.dom.Document; 

// other java stuff
import java.io.File;
import java.util.HashMap;

public class SystemAction extends Action {
    
    /** process a request */
    public Element process (Element message) {
	
	// assume only one request
	Element request = (Element)GSXML.getChildByTagName(message, GSXML.REQUEST_ELEM);
	
	String subaction = request.getAttribute(GSXML.SUBACTION_ATT);
	String lang = request.getAttribute(GSXML.LANG_ATT);
	// get the param list
	Element cgi_param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(cgi_param_list, false);

	Element page = doc_.createElement(GSXML.PAGE_ELEM);
	page.setAttribute(GSXML.LANG_ATT, lang);

	// add the lang stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.DISPLAY_ELEM), true));
	// add the system stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.CONFIGURATION_ELEM), true));

	String coll = (String)params.get(GSCGI.COLLECTION_ARG);

	String to = "";
	if (coll!=null && !coll.equals("")) {
	    to = coll;
	}

	Element mr_request_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_SYSTEM, to, lang);
	mr_request_message.appendChild(mr_request);
	
	Element system = doc_.createElement(GSXML.SYSTEM_ELEM);
	mr_request.appendChild(system);
	
	// will need to change the following if can do more than one system request at once
	if (subaction.equals("c")) { // configure
	    system.setAttribute(GSXML.TYPE_ATT, GSXML.SYSTEM_TYPE_CONFIGURE);
	    String info = (String)params.get(GSCGI.STATUS_SUBSET_ARG);
	    system.setAttribute(GSXML.SYSTEM_SUBSET_ATT, info);
	
	} else {
	    String name = (String)params.get(GSCGI.STATUS_MODULE_NAME_ARG);
	    String type = (String)params.get(GSCGI.STATUS_MODULE_TYPE_ARG);

	    system.setAttribute(GSXML.SYSTEM_MODULE_NAME_ATT, name);
	    system.setAttribute(GSXML.SYSTEM_MODULE_TYPE_ATT, type);
	    
	    if (subaction.equals("d")) { // delete
		system.setAttribute(GSXML.TYPE_ATT, GSXML.SYSTEM_TYPE_DEACTIVATE);

	    } else if (subaction.equals("a")) { // add
		system.setAttribute(GSXML.TYPE_ATT, GSXML.SYSTEM_TYPE_ACTIVATE);
	    } else {
	    // create the default response
	    // for now just have an error
	    System.err.println("SystemAction error: bad subaction type");
	    return null;
	    }
	}

	Element response_message = mr_.process(mr_request_message);
	
	page.appendChild(GSXML.duplicateWithNewName(doc_, (Element)GSXML.getChildByTagName(response_message, GSXML.RESPONSE_ELEM), GSXML.PAGE_RESPONSE_ELEM, true));
	
	// check that the stylesheet is present - cant output the page without one. 
	String stylesheet = GSFile.stylesheetFile(config_, "system.xsl");
	if (stylesheet==null) {
	    System.err.println("SystemAction Error: system stylesheet not found!");
	    return page;
	}
	Document style_doc = converter_.getDOM(new File(stylesheet));
	GSXSLT.absoluteIncludePaths(style_doc, config_);
	return (Element)transformer_.transform(style_doc, page);

	
    }

}
